package App::part;
our $VERSION = '0.07';

=head1 NAME

App::part - split up a single input file into multiple files according to a column value

=head1 SYNOPSIS

This module is just a placeholder for the main program, L<part>

=cut

1;
